import argparse
import os

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='input.faa',metavar='input.faa')


################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def hmmformat(hmmresult):
    # format hmm results
    f1 = open(hmmresult + '2.txt', 'a')
    for line in open(hmmresult, 'r'):
        if str(line)[0] == '#':
            pass
        else:
            line = str(line).replace(' # ', '#')
            while line != str(line).replace('  ', ' '):
                line = str(line).replace('  ', ' ')
            line = str(line).replace(' ', '\t')
            line = str(line).replace('#', ' # ')
            filedir, filename = os.path.split(hmmresult)
            filename = filename.split('.hmm')[0]
            f1.write(filename + '_' + line)
    f1.close()


################################################### Programme #######################################################
hmmformat(os.path.join(args.i,args.f))



import argparse
import os
from Bio import SeqIO


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="file name of your genome", type=str, default='.fa',metavar='.fa')


################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function ########################################################
def check_genome(filename):
    for record in SeqIO.parse(open(filename, 'r'), 'fasta'):
        if 'GCA_' in record.id:
            try:
                str(record.id).split('_')[2]
                return 0
            except IndexError:
                return 1
        else:
            return 0


def formate_genome(filename,Necessary):
    if Necessary == 1:
        os.system('mv '+filename +' '+filename+'.unformat')
        f1=open(filename,'w')
        for record in SeqIO.parse(open(filename+'.unformat', 'r'), 'fasta'):
            newid=str(record.id)+'_'+str(record.description).split(' ')[0]
            f1.write('>' + str(newid) + '\n' + str(record.seq) + '\n')
        f1.close()


################################################### Programme #######################################################
formate_genome(args.i,check_genome(args.i))



import os
from Bio import SeqIO
import argparse
import glob


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-m",
                    help="file name of your cell number mapping file", type=str,
                    default='/scratch/users/anniz44/scripts/argoap/DB/all_KO30_name.list',
                    metavar='all_KO30_name.list')
parser.add_argument("-i",
                    help="input dir of cell number search result", type=str,
                    default='/scratch/users/anniz44/Metagenomes/cellnumber',metavar='.')
parser.add_argument("-f",
                    help="input format of search result", type=str,
                    default='.uscmg.blastx.txt',
                    metavar='.uscmg.blastx.txt')
# optional output setup
parser.add_argument("--r",
                    help="output directory or folder of your results",
                    type=str, default='/scratch/users/anniz44/Metagenomes/summary',
                    metavar='Result')

################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass


################################################### Function ########################################################
def cell_calculate(inputfile,mapping_list):
    output = dict()
    for lines in open(inputfile):
        gene = lines.split('\t')[1]
        maplength = float(lines.split('\t')[3])
        if gene not in mapping_list:
            print(gene,' not in mapping list!')
        else:
            geneID = mapping_list[gene][0]
            genelength = mapping_list[gene][1]
            if geneID not in output:
                output.setdefault(geneID,maplength/genelength)
            else:
                output[geneID] += maplength/genelength
    geneID_count = 0
    for geneID in output:
        geneID_count += 1
    return geneID_count/40.0


################################################### Programme #######################################################
# load all files
inputfiles=glob.glob(os.path.join(args.i,'*'+args.f))

# load cell number mapping file
# gene ID: genotype ID, length
Mapping = dict()
for lines in open(args.m, 'r'):
    Mapping.setdefault(lines.split('\t')[0],
                       [lines.split('\t')[1],float(lines.split('\t')[2].split('\r')[0].split('\n')[0])])

# process cell num calculating
Cellnum=dict()
for filenames in inputfiles:
    filename = os.path.split(filenames)[-1].split(args.f)[0].split('_1')[0].split('_2')[0]
    if filename not in Cellnum:
        Cellnum.setdefault(filename,[])
    Cellnum[filename].append(cell_calculate(filenames,Mapping))

# output results
f1=open(os.path.join(args.r,'cell.num.all.txt'),'w')
for filename in Cellnum:
    f1.write(filename)
    for number in Cellnum[filename]:
        f1.write('\t'+str(number))
    f1.write('\n')

f1.close()



import argparse
from Bio import SeqIO
import os

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='input.faa',metavar='input.faa')
parser.add_argument("-n",
                    help="prefix name for usearch result", type=str, default='.usearch.txt',metavar='.usearch.txt')
parser.add_argument("-r",
                    help="output dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-d",
                    help="extra distance outside the target gene (default: 0 for 0bp)",
                    type=int, default=0,metavar='2000 for 2kbp')
parser.add_argument("-p",
                    help="extract whole sequences or only the hit part \
                    (1: whole; 2: hit), \
                    (default \'1\' for blast search)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=2, type=int)

################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def reverse_complement(dna):
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A','N': 'N'}
    return ''.join([complement[base] for base in dna[::-1]])


def Extractaa(root, searchfile, orffile, resultdir):
    # extract the query aa sequences according to a usearch or diamond alignment output
    # generate a smaller data of potential intI1 or sul1 for blastp search
    # input the query ORF sequences
    AA_seq = dict()
    try:
        f1 = open(os.path.join(resultdir, searchfile + '.aa'), 'w')
        if args.d > 0:
            f2 = open(os.path.join(resultdir, searchfile +
            '.extra' + str(args.d) + '.aa'), 'w')
        try:
            for line in open(os.path.join(resultdir, searchfile), 'r'):
                    AA = str(line).split('\t')[0].split(' ')[0]
                    loci1=int(str(line).split('\t')[6])
                    loci2=int(str(line).split('\t')[7])
                    if args.p == 2:
                        if AA not in AA_seq:
                            AA_seq.setdefault(AA,[[loci1,loci2]])
                        elif [loci1,loci2] not in AA_seq[AA]:
                            AA_seq[AA].append([loci1,loci2])
                    else:
                        AA_seq.setdefault(AA, ['whole'])
            tag = 'fasta'
            if 'fastq' in orffile:
                tag = 'fastq'
            for record in SeqIO.parse(open(os.path.join(root, orffile), 'r'), tag):
                AA = str(record.id)
                total_length = len(str(record.seq))
                if AA in AA_seq:
                    for locus in AA_seq[AA]:
                        if args.p == 2:
                            # extract hit sequences
                            loci1=int(locus[0])
                            loci2=int(locus[1])
                        else:
                            # extract whole sequences
                            loci1 = 1
                            loci2 = len(str(record.seq))
                        if loci1 < loci2:
                            # avoid duplicate ORF
                            f1.write('>%s_%s_%s\n' %(AA,str(loci1-1),str(loci2)) +
                                     str(record.seq)[(loci1-1):loci2] + '\n')
                            if args.d > 0:
                                f2.write('>%s_%s_%s\n' %(AA,str(max(loci1-1-args.d,0)),
                                                         str(min(loci2+args.d,total_length)))+
                                         str(record.seq)[max(loci1-1-args.d,0):
                                         min(loci2+args.d,total_length)] + '\n')
                        else:
                            f1.write('>%s_%s_%s\n' %(AA,str(loci2-1),str(loci1)) +
                                     str(reverse_complement(str(record.seq)[(loci2-1):loci1])) + '\n')
                            if args.d > 0:
                                f2.write('>%s_%s_%s\n' %(AA,str(max(loci2-1-args.d,0)),
                                                         str(min(loci1+args.d,total_length))) +
                                         str(reverse_complement(str(record.seq)[max(loci2-1-args.d,0):
                                         min(loci1+args.d,total_length)])) + '\n')
                    # finish extracting AA
                    AA_seq[AA]=''
        except IOError:
            pass
        f1.close()
    except (IOError):
        print ('Files were missing: ' + orffile)


################################################### Programme #######################################################
Extractaa( args.i, args.f+args.n, args.f, args.r)



import argparse
from Bio import SeqIO
import os

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='input.faa',metavar='input.faa')
parser.add_argument("-n",
                    help="prefix name for usearch result", type=str, default='.usearch.txt',metavar='.usearch.txt')
parser.add_argument("-r",
                    help="output dir", type=str, default='.',metavar='current dir (.)')


################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def Extract16S(root, searchfile, seqfile, resultdir):
    # extract the query aa sequences according to a usearch or diamond alignment output
    # generate a smaller data of potential intI1 or sul1 for blastp search
    # input the query ORF sequences
    Seq_16S = dict()
    try:
        f1 = open(os.path.join(resultdir, searchfile + '.fasta'), 'w')
        for line in open(os.path.join(resultdir, searchfile), 'r'):
            Seq = str(line).split('\t')[0].split(' ')[0]
            if Seq not in Seq_16S:
                Seq_16S.setdefault(Seq, [[int(str(line).split('\t')[6]) - 1, int(str(line).split('\t')[7]) - 1]])
            else:
                for locus in Seq_16S[Seq]:
                    if max(int(str(line).split('\t')[7]) - 1, locus[1]) - \
                    min(int(str(line).split('\t')[6]) - 1, locus[0]) <=2000:
                        # same 16S on a contig
                        locus[0] = min(int(str(line).split('\t')[6]) - 1, locus[0])
                        locus[1] = max(int(str(line).split('\t')[7]) - 1, locus[1])
                    else:
                        # another 16S on a contig
                        Seq_16S[Seq].append([int(str(line).split('\t')[6]) - 1, int(str(line).split('\t')[7]) - 1])
        # screening out the longest 16S
        Genome=[]
        Maxlen=0
        for Seq in Seq_16S:
            for locus in Seq_16S[Seq]:
                if locus[1] - locus[0] > Maxlen:
                    Genome=[Seq,locus[0],locus[1]]
                    Maxlen=locus[1] - locus[0]
        for record in SeqIO.parse(open(os.path.join(root, seqfile), 'r'), 'fasta'):
            if str(record.id) == Genome[0]:
                if Genome[2] - Genome[1] >= 1000:
                    if Genome[2] - Genome[1] <= 2000:
                        f1.write('>' + seqfile.split('.')[0] + '\t' + str(record.id) + '\n' +
                                 str(str(record.seq)[Genome[1] : Genome[2]]) + '\n')
                        flog.write(seqfile+'\toutput\n')
                    else:
                        flog.write(seqfile + '\ttoo_long\n')
                else:
                    flog.write(seqfile + '\ttoo_short\n')
        f1.close()
    except (IOError):
        flog.write(seqfile + '\tfile_missing\n')


################################################### Programme #######################################################
flog = open(os.path.join(args.r, '16S_output.log'), 'a')
Extract16S( args.i, args.f+args.n, args.f, args.r)
flog.close()


import os
from Bio import SeqIO
import argparse
import glob


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-t",
                    help="trait name", type=str, default='ARG',metavar='trait name')
parser.add_argument("-db",
                    help="file name of your input database", type=str,
                    default='Butyrate.pro.aa',metavar='database.aa')
parser.add_argument("-i",
                    help="input dir of Filelist", type=str, default='.',metavar='Filelist.txt')
parser.add_argument("-m",
                    help="mapping file of traits to function", type=str,
                    default='Butyrate.pro.mapping.txt',
                    metavar='Butyrate.pro.mapping.txt')
# optional parameters
parser.add_argument("--fa",
                    help="input format of genome sequence", type=str, default='.fna.add',metavar='.fasta, .fna or .fa')
parser.add_argument("--orf",
                    help="input format of genome orfs", type=str, default='.genes.faa',metavar='.faa')
# optional output setup
parser.add_argument("--r",
                    help="output directory or folder of your results of Traits_WGD.py",
                    type=str, default='Result',metavar='Result')
parser.add_argument("--r16",
                    help="output directory or folder of your 16S sequences of Traits_WGD.py",
                    type=str, default='Result',metavar='Result')
parser.add_argument("--s",
                    help="output directory or folder of your results of traits summary",
                    type=str, default='summary',metavar='summary')
parser.add_argument("-c",
                    help="cutoff for traits",
                    type=float, default=60.0,
                    metavar='60.0')


################################################## Definition ########################################################
args = parser.parse_args()
fasta_format = args.fa
orfs_format = args.orf
if '.add' not in orfs_format:
    orfs_format = orfs_format + '.add'
try:
    os.mkdir(args.s)
except OSError:
    pass


################################################### Function ########################################################
def check_file(listoffiles):
    for files in listoffiles:
        try:
            f1 = open(files,'r')
            return files
        except FileNotFoundError:
            pass


def check_16S(inputfile):
    try:
        file16S= check_file(glob.glob(os.path.join(args.r16 + '/*',
                        inputfile.replace(orfs_format, fasta_format) + '.16S.txt.fasta')))
        Has16S = 0
        if file16S != None:
            for lines in open(file16S,'r'):
                if str(lines)!='':
                    # 16S file not empty
                    Has16S = 1
                break
        if Has16S == 1:
            # merge 16S fasta
            for record in SeqIO.parse(file16S, 'fasta'):
                f16s.write('>'+str(record.id).split('_final')[0]+'\n'+str(record.seq)+'\n')
    except FileNotFoundError:
        pass


def check_traits(inputfile,outputfile_aa,outputfile_aa_2000,outputfile_blast,outputfile_summary,file_subfix,i):
    blastout = check_file(glob.glob(args.r + '/search_output/*/'+ inputfile + '.blast.txt.filter'))
    Hastraits = 0
    if blastout != None:
        for lines in open(blastout,'r'):
            if str(lines)!='':
                # blastout file not empty
                Hastraits = 1
            break
    if Hastraits == 1:
        aaout = blastout + '.aa'
        aaout2000 = blastout + '.extra2000.aa'
        # merge traits fasta
        for record in SeqIO.parse(aaout, 'fasta'):
            if filename.split(file_subfix)[0] in str(record.id):
                outputfile_aa.write('>'+str(record.id)+'\n'+str(record.seq)+'\n')
            else:
                outputfile_aa.write('>'+filename.split(file_subfix)[0] + '_'+ str(record.id)+
                '\n'+str(record.seq)+'\n')
        # merge traits extend 2000 fasta
        if outputfile_aa_2000 != 'None':
            for record in SeqIO.parse(aaout2000, 'fasta'):
                if filename.split(file_subfix)[0] in str(record.id):
                    outputfile_aa_2000.write('>'+str(record.id)+'\n'+str(record.seq)+'\n')
                else:
                    outputfile_aa_2000.write('>'+filename.split(file_subfix)[0] + '_'+ str(record.id)+
                    '\n'+str(record.seq)+'\n')
        # format traits output
        totaltraits=[]
        temp = []
        while i > 0:
            totaltraits.append(0)
            i = i-1
        for lines in open(blastout, 'r'):
            if filename.split(file_subfix)[0] not in lines:
                lines = filename.split(file_subfix)[0] + '_' + lines
            traits_gene=str(lines).split('\t')[1]
            # output blast results
            if Function != dict():
                outputfile_blast.write(Function.get(traits_gene,'None')+'\t'+str(lines))
            else:
                # direct traits output
                outputfile_blast.write(str(lines))
            # calculate copy number of each gene
            if float(lines.split('\t')[2]) >= args.c:
                try:
                    totaltraits[Functionlist[traits_gene]] += 1
                except KeyError: #reference genes not in database
                    pass
        for copy_number in totaltraits:
            temp.append(str(copy_number))
        outputfile_summary.write(inputfile.split(file_subfix)[0]+'\t'+'\t'.join(temp)+'\n')
    else:
        outputfile_summary.write(inputfile.split(file_subfix)[0] + '\tNo_hit\n')


################################################### Programme #######################################################
# load all files
in_dir=args.i
Targetroot=dict()
for root,dirs,files in os.walk(in_dir):
    list_fasta1 = glob.glob(os.path.join(root, '*'+orfs_format))
    if list_fasta1!=[]:
        for files in list_fasta1:
            Targetroot.setdefault(files, orfs_format)


# load traits mapping file
Function = dict()
Functionlist=dict()
genenum=0
for lines in open(args.m,'r'):
    gene = str(lines).split('\t')[0]
    gene_fun = str(lines).split('\t')[1].split('\r')[0].split('\n')[0]
    Function.setdefault(gene,
    gene_fun)
    if gene not in Functionlist:
        Functionlist.setdefault(gene,genenum)
        genenum+=1

# set output files
f16s=open(os.path.join(args.s,args.t+'.all.16S.fasta'),'w')
faa=open(os.path.join(args.s,args.t+'.all.traits.aa.fasta'),'w')
ftraits=open(os.path.join(args.s,args.t+'.all.traits.aa.txt'),'w')
fdna=open(os.path.join(args.s,args.t+'.all.traits.dna.fasta'),'w')
fdna_2000=open(os.path.join(args.s,args.t+'.all.traits.dna.extra2000.fasta'),'w')
ftraits_dna=open(os.path.join(args.s,args.t+'.all.traits.dna.txt'),'w')
fsum_aa = open(os.path.join(args.s,args.t+'.all.traits.aa.summarize.'+str(args.c)+'.txt'),'w')
fsum_aa.write('SampleID')
for functions in Functionlist:
    fsum_aa.write('\t'+str(functions))
fsum_aa.write('\n')
fsum_dna = open(os.path.join(args.s,args.t+'.all.traits.dna.summarize.'+str(args.c)+'.txt'),'w')
fsum_dna.write('SampleID')
for functions in Functionlist:
    fsum_dna.write('\t'+str(functions))
fsum_dna.write('\n')

# process all traits
for filenames in Targetroot:
    filedir, filename = os.path.split(filenames.split('\r')[0].split('\n')[0])
    # check 16S
    check_16S(filename)
    # check and summarize traits
    check_traits(filename,faa,'None',ftraits,fsum_aa,orfs_format,genenum)
    check_traits(filename.replace(orfs_format,fasta_format),fdna,fdna_2000,ftraits_dna,fsum_dna,fasta_format,genenum)


# end of processing all traits
f16s.close()
faa.close()
ftraits.close()
fdna.close()
fdna_2000.close()
ftraits_dna.close()
fsum_aa.close()
fsum_dna.close()



import os
from Bio import SeqIO
import argparse
import glob


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-db",
                    help="file name of your input database",
                    type=str, default='Butyrate.pro.aa',metavar='database.aa')
parser.add_argument("-dbf",
                    help="sequence format of your input database\
                    (1: nucleotide; 2: protein), \
                    (default \'1\' for nucleotide)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
parser.add_argument("-i",
                    help="input dir of MG", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-l",
                    help="input list of MG", type=str, default='None',metavar='rep_metagenomes.txt')
parser.add_argument('-m',
                    help="set the model to predict the results \
                    (1: simple; 2: phylogenetic; 3: advanced), \
                    (default \'1\' for simple)",
                    metavar="1 or 2 or 3",
                    choices=[1, 2, 3],
                    action='store', default=1, type=int)
parser.add_argument('-s',
                    help="set the method to search the your database \
                    (1: blast; 2: hmm), \
                    (default \'1\' for blast search)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
# optional parameters
parser.add_argument("--fa",
                    help="input format of metagenomes sequence", type=str, default='.fasta',metavar='.fasta, .fna or .fa')
#parser.add_argument("--orf",
#                    help="input format of genome orfs", type=str, default='.genes.faa',metavar='.faa')
# optional output setup
parser.add_argument("--r",
                    help="output directory or folder of your results",
                    type=str, default='Result_traits',metavar='Result_traits')
parser.add_argument("--r16",
                    help="output directory or folder of your 16S sequences",
                    type=str, default='Result_16S',metavar='Result_16S')
# optional search parameters
parser.add_argument('--t',
                    help="Optional: set the thread number assigned for running XXX (default 1)",
                    metavar="1 or more", action='store', default=1, type=int)
parser.add_argument('--id',
                    default=75.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based identity cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--ht',
                    default=75.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based hit-length cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--e',
                    default=1e-5, action='store', type=float, metavar='1e-5',
                    help='Optional: set the evalue cutoff for blast or hmm (default is 1e-5)')
# requirement for software calling
parser.add_argument('--u',
                    help="Optional: use two-step method for blast search,"+
                         " \'None\' for using one step, \'usearch\' or \'diamond-blastx\' for using two-step \
                         (complete path to usearch or diamond if not in PATH, \
                         please make sure the search tools can be directly called), (default: \'None\')",
                    metavar="None or usearch",
                    action='store', default='None', type=str)
parser.add_argument('--hmm',
                    help="Optional: complete path to hmmscan if not in PATH,",
                    metavar="/usr/local/bin/hmmscan",
                    action='store', default='hmmscan', type=str)
parser.add_argument('--bp',
                    help="Optional: complete path to blastx or blastn if not in PATH,",
                    metavar="/usr/local/bin/blastx",
                    action='store', default='blastx', type=str)
parser.add_argument('--bwa',
                    help="Optional: complete path to bwa if not in PATH,",
                    metavar="/usr/local/bin/bwa",
                    action='store', default='None', type=str)
################################################## Definition ########################################################
args = parser.parse_args()
fasta_format = args.fa
#orfs_format = args.orf
try:
    os.mkdir(args.r)
except OSError:
    pass
workingdir=os.path.abspath(os.path.dirname(__file__))


################################################### Function ########################################################
def addname(filedir, file_name):
    Fasta_name = open(os.path.join(filedir,file_name), 'r')
    f = open(os.path.join(filedir,file_name + '.add'), 'w')
    in_dir, input_file = os.path.split(file_name)
    for record in SeqIO.parse(Fasta_name, 'fasta'):
        if len(str(record.seq).replace(' ',''))>0:
            # remove empty ORF sequences, otherwise there could be a problem for blast, usearch and diamond
            f.write('>'+str(input_file)+'_'+str(record.id) + '\t' + str(record.description).replace('\t', ' ') + '\n' + str(
                str(record.seq)) + '\n')
    f.close()


def search(roottemp,filename):
    # search the database for each file
    cmds = ''
    cmds += "#!/bin/bash \nmodule add c3ddb/blast+/2.7.1 \n"
    # using blastp
    if args.s == 1:
        if args.u != 'None':
            # two-step search
            Usearch=0
            for root, dirs, files in os.walk(args.r + '/usearch'):
                try:
                    ftry = open(os.path.join(root, filename + '.usearch.txt.aa'), 'r')
                    searchfile = os.path.join(root, filename + '.usearch.txt.aa')
                    Usearch=1
                except IOError:
                    pass
            if Usearch == 0:
                if 'usearch' in args.u:
                    # Start search target genes by usearch
                    if args.dbf == 1:
                        cmds += args.u + " -ublast " + os.path.join(roottemp, filename) + \
                                " -db " + args.db + ".udb -strand both -evalue 1e-2 -accel 0.5 -blast6out " \
                                + os.path.join(args.r + '/usearch/' + str(int(i/10000)), filename + '.usearch.txt') + \
                                " -threads " + str(int(i_max)) + " \n"
                    else:
                        cmds += args.u + " -ublast " + os.path.join(roottemp, filename) + \
                                " -db " + args.db + ".udb -evalue 1e-2 -accel 0.5 -blast6out " \
                                + os.path.join(args.r + '/usearch/' + str(int(i/10000)), filename + '.usearch.txt') + \
                                " -threads " + str(int(i_max)) + " \n"
                elif "diamond" in args.u:
                    # Start search target genes by diamond!
                    cmds += args.u.replace('-',' ') + " --query " + os.path.join(roottemp, filename) + \
                            " --db " + args.db + ".dmnd --out " + os.path.join(args.r + '/usearch/' + str(int(i/10000)),
                                                                          filename + '.usearch.txt') + \
                            " --outfmt 6 --max-target-seqs 1 --evalue " + str(args.e) + " --threads " + str(
                        int(i_max)) + " \n"
                elif 'hs-blastn' in args.u:
                    # Start search target genes by hs-blastn
                    if args.dbf == 1:
                        cmds += "%s align -db %s -window_masker_db %s.counts.obinary -query %s -out %s -outfmt 6 -evalue %s -num_threads %s\n"\
                        %(args.u,args.db,args.db, os.path.join(roottemp, filename) ,
                          os.path.join(args.r + '/usearch/' + str(int(i/10000)), filename + '.usearch.txt'),
                          str(args.e),str(int(i_max)))
                cmds += 'python '+ workingdir +'/Extract.MG.py -p 1 -i ' + roottemp + ' -f ' + filename + ' -n .usearch.txt -r ' + args.r + '/usearch/' + str(
                    int(i / 10000)) + ' \n'
                searchfile = os.path.join(args.r + '/usearch/' + str(int(i/10000)), filename + '.usearch.txt.aa')
        else:
            # one-step search
            searchfile = os.path.join(roottemp, filename)
        if args.bp != 'None':
            # blast search
            Blastsearch = 0
            for root, dirs, files in os.walk(args.r + '/search_output'):
                try:
                    ftry = open(os.path.join(root, filename + '.blast.txt'), 'r')
                    Blastsearch = 1
                except IOError:
                    pass
            if Blastsearch == 0:
                # for short metagenomic reads
                cmds += str(args.bp) +" -query " + str(searchfile) + " -db " + args.db + " -out " + args.r + '/search_output/'+str(int(i/10000))+ \
                         "/"+filename+".blast.txt  -outfmt 6 -evalue "+str(args.e)+" -num_threads " + \
                        str(int(i_max)) + " \n"
                # fiter blast result
            Blastsearchfilter = 0
            for root, dirs, files in os.walk(args.r + '/search_output'):
                try:
                    ftry = open(os.path.join(root, filename + '.blast.txt.filter'), 'r')
                    Blastsearchfilter = 1
                except IOError:
                    pass
            if Blastsearchfilter == 0:
                cmds += 'python '+ workingdir +'/Filter.MG.py -i ' + args.r + '/search_output/'+str(int(i/10000))+' -f ' + filename + '.blast.txt ' +\
                        '-db ' + args.db + ' -dbf ' + str(args.dbf) + ' -s ' + str(args.s) + ' --ht ' + str(args.ht) + ' --id ' + str(args.id) + \
                        ' --e ' + str(args.e) + ' \n'
                cmds += 'python '+ workingdir +'/Extract.MG.py -p 1 -i ' + roottemp + ' -f ' + filename + ' -n .blast.txt.filter -r ' + args.r + '/search_output/'+str(int(i/10000))+' \n'
    else:
        # hmmsearch
        Blastsearch = 0
        for root, dirs, files in os.walk(args.r + '/search_output'):
            try:
                ftry = open(os.path.join(root, filename + '.hmm'), 'r')
                Blastsearch = 1
            except IOError:
                pass
        if Blastsearch == 0:
            cmds = args.hmm + ' --tblout ' + os.path.join(args.r + '/search_output/'+str(int(i/10000)), str(
                filename) + '.hmm') +  ' --cpu ' + str(int(i_max)) + ' -E ' \
                  + str(args.e) + ' ' +args.db + ' '+ os.path.join(roottemp, filename) + ' \n'
            cmds += 'python '+ workingdir +'/Format.MG.py -i ' + args.r + '/search_output/'+str(int(i/10000)) + ' -f ' + str(
                    filename) + '.hmm \n'
    #bowtie alignment
    if args.bwa != 'None':
        tempinput = os.path.join(args.r + '/search_output/'+str(int(i/10000)),
                                  filename + '.blast.txt.filter.aa')
        tempbamoutput = os.path.join(args.r + '/bwa/' + str(int(i / 10000)), str(
                filename) + '.blast.txt.filter.aa')
        cmds += args.bwa + ' mem %s %s |samtools view -S -b >%s.bam \nsamtools sort %s.bam -o %s.sorted.bam | samtools index %s.sorted.bam\n' % (
            args.db, tempinput,
            tempbamoutput, tempbamoutput, tempbamoutput,tempbamoutput)
        cmds += 'bcftools mpileup -Ou -f %s %s.sorted.bam  | bcftools call -mv > %s.raw.vcf' %(args.db, tempbamoutput,tempbamoutput)
        cmds += '\nbcftools filter -s LowQual -e \'%s || DP>100\' %s.raw.vcf > %s.flt.vcf \n' %('QUAL<20',tempbamoutput,tempbamoutput)
        tempinput = tempinput.replace('_1' + fasta_format, '_2' + fasta_format)
        tempbamoutput = os.path.join(args.r + '/bwa/' + str(int(i / 10000)), str(
            filename.replace('_1' + fasta_format, '_2' + fasta_format)) + '.blast.txt.filter.aa')
        cmds += args.bwa + ' mem %s %s |samtools view -S -b >%s.bam \nsamtools sort %s.bam -o %s.sorted.bam\nsamtools index %s.sorted.bam\n' % (
            args.db, tempinput,
            tempbamoutput, tempbamoutput, tempbamoutput, tempbamoutput)
        cmds += 'bcftools mpileup -Ou -f %s %s.sorted.bam  | bcftools call -mv > %s.raw.vcf' % (
        args.db, tempbamoutput, tempbamoutput)
        cmds += '\nbcftools filter -s LowQual -e \'%s || DP>100\' %s.raw.vcf > %s.flt.vcf \n' % (
        'QUAL<20', tempbamoutput, tempbamoutput)
    # 16S extraction
    Search16s = 0
    for root, dirs, files in os.walk(args.r16):
        try:
            ftry = open(os.path.join(root, filename + '.16S.txt'), 'r')
            Search16s = 1
        except IOError:
            pass
    if Search16s == 0:
        # Start search 16S by usearch
        cmds += 'python '+ workingdir +'/undone.MG.py -i '+ os.path.join(roottemp.replace('_faa','_fasta'), filename+' \n')
        # with usearch
        #cmds += args.u + " -usearch_global " + os.path.join(roottemp.replace('_faa','_fasta'), filename) + \
        cmds += "/scratch/users/anniz44/bin/miniconda3/bin/usearch11.0.667_i86linux32 -usearch_global " + os.path.join(roottemp.replace('_faa', '_fasta'), filename) + \
                        " -db "+ workingdir +"/../database/85_otus.fasta.all.V4_V5.fasta.udb -strand plus -id 0.7 -evalue 1e-1 -blast6out " \
                + os.path.join(args.r16+'/' + str(int(i/10000)), filename+ '.16S.txt') + \
                " -threads " + str(int(i_max)) + " \n"
        #cmds += str(args.bp).replace('blastx','blastn') +" -query " + os.path.join(roottemp.replace('_faa','_fasta'), filename)\
         #       + " -db "+ workingdir +"/../database/85_otus.fasta.all.V4_V5.fasta -out " + os.path.join(args.r16+'/' + str(int(i/10000)), filename+ '.16S.txt') +\
        #"  -outfmt 6   -evalue 1e-5 -num_threads " + \
         #           str(int(i_max)) + " -task blastn-short \n"
        cmds += 'python '+ workingdir +'/Extract.16S.MG.py -i ' + roottemp.replace('_faa','_fasta') + ' -f ' + \
                filename + ' -n .16S.txt -r ' + args.r16 + '/' + str(
            int(i / 10000)) + ' \n'
        #cmds += 'bayerstraits_16s --m mafft --ft fasttree --th 30 -t /scratch/users/anniz44/GHM/data/genome/summary/GHM.otutable -s /scratch/users/anniz44/GHM/data/genome/summary/all.16S.fasta -top 4018  -r /scratch/users/anniz44/GHM/data/genome/summary/butyrate_predict_hit60_id60'
    return cmds


################################################### Programme #######################################################
# load all WGD
flist=open('Filelist.txt','w')
in_dir=args.i
Targetroot=dict()

Targetlist=[]
if args.l != 'None':
    for lines in open(args.l,'r'):
        Targetlist.append(lines.split('\r')[0].split('\n')[0])

for root,dirs,files in os.walk(in_dir):
    list_fasta1 = glob.glob(os.path.join(root, '*'+fasta_format))
    if list_fasta1!=[]:
        for files in list_fasta1:
            if args.l == 'None' or any(targets in files for targets in Targetlist):
                Targetroot.setdefault(files, fasta_format)


# search the database in all genomes
i=0
#os.system("rm -rf *.sh \n")
#os.system("rm -rf nohup.sh \n")
#i_max=max(int(args.t)/len(Targetroot),1)
try:
    os.mkdir('subscripts')
except OSError:
    pass
try:
    os.mkdir(args.r + '/search_output')
except OSError:
    pass
try:
    os.mkdir(args.r + '/usearch')
except OSError:
    pass
try:
    os.mkdir(args.r16)
except OSError:
    pass
if args.bwa != 'None':
    try:
        os.mkdir(args.r + '/bwa/')
    except OSError:
        pass
i_max=40
for files in Targetroot:
    if Targetroot[files]!='None':
        f1 = open(os.path.join('subscripts',str(i%int(args.t)) + '.sh'), 'a')
        i +=1
        roottemp, filename = os.path.split(files)
        flist.write(str(files) + '\n')
        try:
            os.mkdir(args.r + '/search_output/'+str(int(i/10000)))
        except OSError:
            pass
        try:
            os.mkdir(args.r + '/usearch/'+str(int(i/10000)))
        except OSError:
            pass
        try:
            os.mkdir(args.r16+'/'+str(int(i/10000)))
        except OSError:
            pass
        if args.bwa != 'None':
            try:
                os.mkdir(args.r + '/bwa/' + str(int(i / 10000)))
            except OSError:
                pass
        # add filename to orf
        #try:
        #    ftry = open(os.path.join(roottemp, str(filename) + ".add"), 'r')
        #except IOError:
        #    addname(roottemp, str(filename))
        #filename = filename + ".add"
        # search the database in WGD
        cmds = search(roottemp, filename)
        f1.write(cmds)
        f1.close()
flist.close()



import argparse
import os
from Bio import SeqIO


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-db",
                    help="file name of your input database",
                     type=str, default='Butyrate.pro.aa',metavar='database.aa')
parser.add_argument("-dbf",
                    help="sequence format of your input database\
                    (1: nucleotide; 2: protein), \
                    (default \'1\' for nucleotide)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='.blast.txt',metavar='.blast.txt')
parser.add_argument('-s',
                    help="set the method to search the your database \
                    (1: blast; 2: hmm), \
                    (default \'1\' for blast search)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
# optional parameters
# optional search parameters
parser.add_argument('--id',
                    default=80.0, action='store', type=float, metavar='80.0',
                    help='Optional: set the amno acid based identity cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--ht',
                    default=80.0, action='store', type=float, metavar='80.0',
                    help='Optional: set the amno acid based hit-length cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--e',
                    default=1e-5, action='store', type=float, metavar='1e-5',
                    help='Optional: set the evalue cutoff for blast or hmm (default is 1e-5)')

################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def Calculate_length(file_name):
    DB_length=dict()
    try:
        for lines in open(file_name + '.length', 'r'):
            DB_length.setdefault(str(lines.split('\t')[0]), float(str(lines.split('\t')[-1]).replace('\n','')))
    except (IOError):
        Fasta_name = open(file_name, 'r')
        f = open(file_name + '.length', 'w')
        for record in SeqIO.parse(Fasta_name, 'fasta'):
            f.write(str(record.id) + '\t'  + str(
                len(record.seq)) + '\n')
            DB_length.setdefault(str(record.id), len(str(record.seq)))
        f.close()
    return DB_length

def compare_blast(All_hit_set,newline):
    ID2 = float(str(newline).split('\t')[2])
    Loci2 = (float(str(newline).split('\t')[6]) + float(str(newline).split('\t')[7])) / 2.0
    for oldlines in All_hit_set:
        ID1=float(str(oldlines).split('\t')[2])
        Loci1=(float(str(oldlines).split('\t')[6])+float(str(oldlines).split('\t')[7]))/2.0
        if abs(Loci1 - Loci2) > 500 or ID1 == ID2:
            All_hit_set.append(newline)
            break
        elif ID1 > ID2:
            break
        elif ID2 > ID1:
            All_hit_set.append(newline)
            All_hit_set.remove(oldlines)
            break
    return All_hit_set


def blast_list(file, Cutoff_identity,Cutoff_hitlength):
    f1=open(file+'.filter','w')
    All_hit = dict()
    for line in open(file, 'r'):
        if float(str(line).split('\t')[2]) >= Cutoff_identity:
            try:
                if float(str(line).split('\t')[3]) >= Cutoff_hitlength * float(
                        DB_length.get(str(line).split('\t')[1],0.0)) / 100:
                    if str(line).split('\t')[0] not in All_hit:
                        All_hit.setdefault(str(line).split('\t')[0],
                        [line])
                    else:
                        All_hit[str(line).split('\t')[0]] = compare_blast(All_hit[str(line).split('\t')[0]],line)
            except TypeError:
                print (str(line).split('\t')[1],' has no length')
    for genes in All_hit:
        for hits in All_hit[genes]:
            f1.write(hits)
    f1.close()


################################################### Programme #######################################################
if args.s == 1:
    DB_length = dict()
    if args.ht > 0.0:
        DB_length=Calculate_length(args.db)
    blast_list(os.path.join(args.i, args.f), float(args.id), float(args.ht))
else:
    pass



import os
from Bio import SeqIO
import argparse
import glob


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-db",
                    help="file name of your input database",
                    type=str, default='Butyrate.pro.aa',metavar='database.aa')
parser.add_argument("-dbf",
                    help="sequence format of your input database\
                    (1: nucleotide; 2: protein), \
                    (default \'1\' for nucleotide)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
parser.add_argument("-i",
                    help="input dir of WGD", type=str, default='.',metavar='current dir (.)')
parser.add_argument('-m',
                    help="set the model to predict the results \
                    (1: simple; 2: phylogenetic; 3: advanced), \
                    (default \'1\' for simple)",
                    metavar="1 or 2 or 3",
                    choices=[1, 2, 3],
                    action='store', default=1, type=int)
parser.add_argument('-s',
                    help="set the method to search the your database \
                    (1: blast; 2: hmm), \
                    (default \'1\' for blast search)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
# optional parameters
parser.add_argument("--fa",
                    help="input format of genome sequence", type=str,
                    default='.fna.add',metavar='.fasta, .fna or .fa')
parser.add_argument("--orf",
                    help="input format of genome orfs", type=str,
                    default='.genes.faa',metavar='.faa')
# optional output setup
parser.add_argument("--r",
                    help="output directory or folder of your results",
                    type=str, default='Result',metavar='Result')
parser.add_argument("--r16",
                    help="output directory or folder of your 16S sequences",
                    type=str, default='Result',metavar='Result')
# optional search parameters
parser.add_argument('--t',
                    help="Optional: set the thread number assigned for running XXX (default 1)",
                    metavar="1 or more", action='store', default=1, type=int)
parser.add_argument('--id',
                    default=75.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based identity cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--ht',
                    default=75.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based hit-length cutoff for blast (default is 80.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--e',
                    default=1e-2, action='store', type=float, metavar='1e-5',
                    help='Optional: set the evalue cutoff for blast or hmm (default is 1e-5)')
# requirement for software calling
parser.add_argument('--u',
                    help="Optional: use two-step method for blast search,"+
                         " \'None\' for using one step, \'usearch\' or \'diamond\' for using two-step \
                         (complete path to usearch or diamond if not in PATH, \
                         please make sure the search tools can be directly called), (default: \'None\')",
                    metavar="None or usearch",
                    action='store', default='None', type=str)
parser.add_argument('--hmm',
                    help="Optional: complete path to hmmscan if not in PATH,",
                    metavar="/usr/local/bin/hmmscan",
                    action='store', default='hmmscan', type=str)
parser.add_argument('--bp',
                    help="Optional: complete path to blastp or blastn if not in PATH, \'None\' for no blast search",
                    metavar="/usr/local/bin/blastp",
                    action='store', default='blastp', type=str)
parser.add_argument('--bwa',
                    help="Optional: complete path to bwa if not in PATH,",
                    metavar="/usr/local/bin/bwa",
                    action='store', default='None', type=str)

################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass
try:
    os.mkdir('subscripts')
except OSError:
    pass
workingdir=os.path.abspath(os.path.dirname(__file__))

################################################### Programme #######################################################
# set the search for database type
diamond_set = args.u
if args.dbf == 1:
    blast_set=args.bp.replace('blastp','blastn')
    if 'diamond' in args.u:
        print('Diamond cannot use dna database\nDirect use blast!')
        diamond_set = 'None'
else:
    blast_set=args.bp.replace('blastn','blastp')
    diamond_set=args.u.replace('diamond','diamond-blastp')
# search the database in all genomes
cmds = 'python '+ workingdir +'/scripts/Search.WG.py -i ' + args.i  +\
                ' -db ' + args.db + ' -dbf ' + str(args.dbf) + ' -s ' + str(args.s) + ' --r ' + str(args.r) + ' --t ' + str(args.t) + \
                ' --u ' + str(diamond_set) + ' --hmm ' + str(args.hmm) + ' --bp ' + str(blast_set) + \
                ' --ht ' + str(args.ht) + ' --id ' + str(args.id) + ' --fa ' + str(args.fa) + \
                ' --e ' + str(args.e) + ' --orf ' + str(args.orf) + ' --r16 ' + str(args.r16)+\
        ' --bwa ' + str(args.bwa) + ' \n'
#print(cmds)
os.system(cmds)

# run all bash
list_of_files = glob.glob('subscripts/*.sh')
f1 = open("all.sh", 'w')
f1.write("#!/bin/bash \nmodule add c3ddb/blast+/2.7.1 \n")
# make all database
if args.s == 1:
    if args.bwa != 'None':
        try:
            ftest=open(args.db+'.bwt','r')
        except IOError:
            f1.write('bwa index %s \n' % (args.db))
    if args.dbf == 1:
        try:
            ftest = open(args.db + '.nin', 'r')
        except IOError:
            f1.write(os.path.join(os.path.split(args.bp)[0],'makeblastdb -in %s -dbtype nucl\n' % (args.db)))
            if args.u != 'None':
                if 'usearch' in args.u:
                    f1.write(os.path.join(os.path.split(args.u)[0], 'usearch -makeudb_usearch %s -output %s.udb\n' % (args.db,args.db)))
                elif 'hs-blastn' in args.u:
                    f1.write(os.path.join(os.path.split(args.u)[0],
                                          'windowmasker -in %s -infmt blastdb -mk_counts -out %s.counts\n' % (args.db, args.db)))
                    f1.write(os.path.join(os.path.split(args.u)[0],
                                          'windowmasker -in %s.counts -sformat obinary -out %s.counts.obinary -convert\n' % (
                                          args.db, args.db)))
                    f1.write(os.path.join(os.path.split(args.u)[0],
                                          'hs-blastn index %s\n' % (
                                          args.db)))
    else:
        try:
            ftest = open(args.db + '.pin', 'r')
        except IOError:
            f1.write(os.path.join(os.path.split(args.bp)[0],'makeblastdb -in %s -dbtype prot\n' % (args.db)))
            if args.u != 'None':
                if 'usearch' in args.u:
                    f1.write(os.path.join(os.path.split(args.u)[0], 'usearch -makeudb_usearch %s -output %s.udb\n' % (args.db,args.db)))
                elif 'diamond' in args.u:
                    f1.write(os.path.join(os.path.split(args.u)[0],
                                          'diamond makedb --in %s -d %s.dmnd\n' % (args.db, args.db)))
# makedb for 16S
try:
    ftest = open(workingdir +'/database/85_otus.fasta.udb', 'r')
except IOError:
    if args.u != 'None':
        if 'usearch' in args.u:
            f1.write(os.path.join(os.path.split(args.u)[0],
                                  'usearch -makeudb_usearch %s -output %s.udb\n' % (workingdir +'/database/85_otus.fasta', workingdir +'/database/85_otus.fasta')))
try:
    ftest = open(workingdir +'/database/85_otus.fasta.counts', 'r')
except IOError:
    if args.u != 'None':
            if 'hs-blastn' in args.u:
                f1.write(os.path.join(os.path.split(args.u)[0],
                                      'windowmasker -in %s -infmt blastdb -mk_counts -out %s.counts\n' % (
                                          workingdir +'/database/85_otus.fasta', workingdir +'/database/85_otus.fasta')))
                f1.write(os.path.join(os.path.split(args.u)[0],
                                      'windowmasker -in %s.counts -sformat obinary -out %s.counts.obinary -convert\n' % (
                                          workingdir +'/database/85_otus.fasta', workingdir +'/database/85_otus.fasta')))
                f1.write(os.path.join(os.path.split(args.u)[0],
                                      'hs-blastn index %s\n' % (
                                          workingdir +'/database/85_otus.fasta')))
# bash for all subscripts
for file_name in list_of_files:
    if all(keys not in file_name for keys in ['SearchWG.sh','all.sh']):
        f1.write("sbatch -p defq -c 40 -t 5-00:00:00 --mem=100000 -J " + str(file_name) + "_traits -o " + str(file_name) + ".out -e " + str(file_name) + ".err " + str(file_name) +" \n")
    #f1.write("nohup sh " + str(file_name) + '  & \n')
f1.close()
#os.system("nohup sh all.sh & \n")



import glob
import os
import argparse
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="input dir of traits", type=str,
                    default='/scratch/users/anniz44/scripts/traits_search_MG/Result_traits/search_output/*',
                    metavar='current dir (.)')
parser.add_argument("-i16",
                    help="input dir of traits", type=str,
                    default='/scratch/users/anniz44/scripts/traits_search_MG/Result_16S/*',
                    metavar='current dir (.)')
parser.add_argument("-o",
                    help="output directory",
                    type=str, default='/scratch/users/anniz44/scripts/traits_search_MG/summary', metavar='output')
parser.add_argument("-f",
                    help="input format of traits",
                    type=str, default='.fasta.blast.txt.filter', metavar='.fasta.blast.txt.filter')
parser.add_argument("-f16",
                    help="input format of 16S",
                    type=str, default='.fasta.16S.txt', metavar='.fasta.16S.txt')
parser.add_argument("-l",
                    help="traits length",
                    type=str, default='/scratch/users/anniz44/scripts/traits_search/Butyrate.pro.aa.length',
                    metavar='Butyrate.pro.aa.length')
parser.add_argument("-tf",
                    help="traits function",
                    type=str, default='/scratch/users/anniz44/scripts/traits_search_MG/butyrate.pro.mapping.txt',
                    metavar='butyrate.pro.mapping.txt')
parser.add_argument("-m",
                    help="habitat metadata",
                    type=str, default='/scratch/users/anniz44/scripts/traits_search_MG/MGD_meta_new.txt',
                    metavar='MGD_meta_new.txt')
parser.add_argument("-b",
                    help="blastresults",
                    type=str, default='/scratch/users/anniz44/scripts/traits_search_MG/alltraits.to.alltraits.90.0.dedupli.txt',
                    metavar='alltraits.to.alltraits.90.0.dedupli.txt.usearch')
parser.add_argument("-c",
                    help="cutoff for traits",
                    type=float, default=90.0,
                    metavar='90.0')


################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.o)
except OSError:
    pass
try:
    os.mkdir(args.o+'/similaritynew')
except OSError:
    pass

################################################## Function ########################################################
def check_file(listoffiles):
    for files in listoffiles:
        try:
            f1 = open(files,'r')
            return files
        except FileNotFoundError:
            pass


def copycal(traitsfile,file16S,i):
    copy1 = copytraits(traitsfile,i)
    copy2 = copy16S(file16S)
    temp = []
    temp.append(str(copy2))
    for traitscopy in copy1:
        temp.append(str(traitscopy))
        if copy2 > 0:
            temp.append(str(traitscopy / copy2))
        else:
            print(file16S, copy2,traitsfile)
            temp.append(str(0))
    return temp

def findfunction(gene):
    return Functionlist[Function[gene]]


def copytraits(traitsfile,i):
    #totaltraits=[0,0,0,0,0]
    totaltraits=[]
    while i > 0:
        totaltraits.append(0)
        i = i-1
    for lines in open(traitsfile,'r'):
        if float(lines.split('\t')[2]) >= args.c:
            gene = lines.split('\t')[1]
            totaltraits[findfunction(gene)] += float(lines.split('\t')[3])/Length[lines.split('\t')[1]]
    return totaltraits


def copy16S(file16S):
    total16S=0
    try:
        for lines in open(file16S,'r'):
            total16S += float(lines.split('\t')[3])/1522.0
    except FileNotFoundError:
        pass
    return total16S


def traits_filter(blastfile, aafile):
    f1 = open(aafile+'.'+str(args.c)+'.filter','w')
    filtered = []
    for lines in open(blastfile,'r'):
        if float(lines.split('\t')[2]) >= args.c:
            filtered.append(lines.split('\t')[0])
    for record in SeqIO.parse(aafile, "fasta"):
        if record.id in filtered:
            f1.write('>'+str(record.id)+'\n'+str(record.seq)+'\n')
    f1.close()


def traits_dedupli(aallseqlist,blastfile):
    tempseq=[]
    for lines in open(blastfile,'r'):
        aallseqlist[lines.split('\t')[0]] = Function[lines.split('\t')[1]]


def traits_dedupli2(aseqlist, aallseqlist,blastfile,aafile):
    tempseq=[]
    f1 = open(aafile + '.dedupli', 'w')
    for record in SeqIO.parse(aafile, "fasta"):
        #aallseqlist.setdefault(str(record.id),'None')
        if str(record.seq) not in tempseq:
            #aseqlist.setdefault(str(record.id),'None')
            tempseq.append(str(record.seq))
            f1.write('>' + str(record.id) + '\n' + str(record.seq) + '\n')
    f1.close()


def habitat_tag(habitattype):
    if habitattype in ['fecal_animal','fecal_human']:
        return 'Fecal'
    elif habitattype in ['wwtp']:
        return 'WWTP'
    else:
        return 'Nature'

def habitat_pair(habitattype1,habitattype2,tagpair):
    tag1=habitat_tag(habitattype1)
    tag2 = habitat_tag(habitattype2)
    if tag1 == tag2:
        return tag1
    else:
        if tag1 + '__' + tag2 in tagpair:
            return tag1 + '__' + tag2
        elif tag2 + '__' + tag1 in tagpair:
            return tag2 + '__' + tag1
        else:
            tagpair.append(tag1 + '__' + tag2)
            return tag1 + '__' + tag2

def swap(key):
    # for subtype
    if 'wwtp_' in key:
        return 'wwtp_'+key.split('wwtp_')[1].split('_')[0]+'_'+key.replace('wwtp_'+key.split('wwtp_')[1].split('_')[0],'')
    elif 'natural_' in key:
        try:
            return 'natural_'+key.split('natural_')[1].split('_')[0]+'_'+key.split('natural_')[1].split('_')[1]+'_'+key.replace('natural_'+key.split('natural_')[1].split('_')[0]+'_'+key.split('natural_')[1].split('_')[1],'')
        except IndexError:
            return 'natural_'+key.split('natural_')[1].split('_')[0]+'_'+key.replace('natural_'+key.split('natural_')[1].split('_')[0],'')
    elif 'fecal_animal_' in key:
        return 'fecal_animal_'+key.split('fecal_animal_')[1].split('_')[0]+'_'+key.replace('fecal_animal_'+key.split('fecal_animal_')[1].split('_')[0],'')
    else:
        return key


def swap2(key):
    # for type
    if 'wwtp' in key:
        return 'wwtp'+'_'+key.replace('wwtp','')
    elif 'natural_' in key:
        return 'natural_'+key.split('natural_')[1].split('_')[0]+'_'+key.replace('natural_'+key.split('natural_')[1].split('_')[0],'')
    elif 'fecal_animal' in key:
        return 'fecal_animal'+'_'+key.replace('fecal_animal','')
    else:
        return key


def swap3(key):
    return key.replace('__','_').replace('fecal_animal','Animal_feces').replace('fecal_human','Human_feces').replace('natural_','Nature_').replace('wwtp','WWTP')


def traits_habitat(blastresult,allseqlist):
    habitatset=[]
    tagpair = []
    for lines in open(blastresult):
            if allseqlist[lines.split('\t')[0]] == allseqlist[lines.split('\t')[1]]:
                MG1 = lines.split('\t')[0].split('.')[0]
                MG2 = lines.split('\t')[1].split('.')[0]
                if lines.split('\t')[0] != lines.split('\t')[1]:
                    if Meta[MG1][0] == Meta[MG2][0]:
                        #file1 = open(os.path.join(args.o+'/similaritynew',Meta[MG1][0]+'.intrasubtype.similaritynew'),'a')
                        #file1.write(str(lines.split('\t')[2])+'\n')
                        #file1.close()
                        file1 = open(os.path.join(args.o+'/similaritynew','Intrasubtype.similaritynew'),'a')
                        file1.write(Meta[MG1][1]+'\t'+Meta[MG1][0]+'\t'+str(lines.split('\t')[2]) + '\tIntrasubtype\n')
                        file1.close()
                    else:
                        #file1 = open(os.path.join(args.o+'/similaritynew', Meta[MG1][0] + '_' + Meta[MG2][0] +'.intersubtype.similaritynew'), 'a')
                        #file1.write(str(lines.split('\t')[2]) + '\n')
                        #file1.close()
                        if Meta[MG1][0] + '_' + Meta[MG2][0] in habitatset:
                            habitatpair = Meta[MG1][0] + '_' + Meta[MG2][0]
                        elif Meta[MG2][0] + '_' + Meta[MG1][0] in habitatset:
                            habitatpair = Meta[MG2][0] + '_' + Meta[MG1][0]
                        else:
                            habitatpair = Meta[MG1][0] + '_' + Meta[MG2][0]
                            habitatset.append(habitatpair)
                        file1 = open(os.path.join(args.o+'/similaritynew', 'Intersubtype.similaritynew'), 'a')
                        file1.write(habitat_pair(Meta[MG1][1],Meta[MG2][1],tagpair)+'\t'+swap3(swap(habitatpair)) + '\t' + str(lines.split('\t')[2]) + '\tIntersubtype\n')
                        file1.close()
                    if Meta[MG1][1] == Meta[MG2][1]:
                        #file1 = open(os.path.join(args.o+'/similaritynew',Meta[MG1][1]+'.intratype.similaritynew'),'a')
                        #file1.write(str(lines.split('\t')[2])+'\n')
                        #file1.close()
                        file1 = open(os.path.join(args.o+'/similaritynew','Intratype.similaritynew'),'a')
                        file1.write(Meta[MG1][1] + '\t' + Meta[MG1][1] + '\t' + str(lines.split('\t')[2])+'\tIntratype\n')
                        file1.close()
                    else:
                        #file1 = open(os.path.join(args.o+'/similaritynew', Meta[MG1][1] + '_' + Meta[MG2][1] +'.intertype.similaritynew'), 'a')
                        #file1.write(str(lines.split('\t')[2]) + '\n')
                        #file1.close()
                        if Meta[MG1][1] + '_' + Meta[MG2][1] in habitatset:
                            habitatpair = Meta[MG1][1] + '_' + Meta[MG2][1]
                        elif Meta[MG2][1] + '_' + Meta[MG1][1] in habitatset:
                            habitatpair =Meta[MG2][1] + '_' + Meta[MG1][1]
                        else:
                            habitatpair = Meta[MG1][1] + '_' + Meta[MG2][1]
                            habitatset.append(habitatpair)
                        file1 = open(os.path.join(args.o+'/similaritynew', 'Intertype.similaritynew'), 'a')
                        file1.write(habitat_pair(Meta[MG1][1],Meta[MG2][1],tagpair)+'\t'+swap3(swap2(habitatpair))+'\t'+str(lines.split('\t')[2]) + '\tIntertype\n')
                        file1.close()


################################################## Programme ########################################################
# load traits length
Length=dict()
for lines in open(args.l,'r'):
    Length.setdefault(lines.split('\t')[0],
                      float(lines.split('\t')[-1].split('\r')[0].split('\n')[0]))

# load traits function
Function = dict()
Functionlist=dict()
genenum=0
for lines in open(args.m,'r'):
    gene = str(lines).split('\t')[0]
    gene_fun = str(lines).split('\t')[1].split('\r')[0].split('\n')[0]
    Function.setdefault(gene,
    gene_fun)
    if gene_fun not in Functionlist:
        Functionlist.setdefault(gene_fun,genenum)
        genenum+=1

# load metadata
Meta=dict()
for lines in open(args.m,'r'):
    Meta.setdefault(lines.split('\t')[-1].split('.')[0].split('\r')[0].split('\n')[0],
                    [lines.split('\t')[2],lines.split('\t')[1]])

# load all MG results
alltraitsfile = glob.glob(os.path.join(args.i,'*'+args.f))
fout = open(os.path.join(args.o,'traits_copy_per_16S.'+str(args.c)+'.txt'),'w')
fout.write('SampleID\t16Scopy\t')
for functions in Functionlist:
    fout.write(str(functions)+'copy\t'+str(functions)+'copyper16S\t')
fout.write('Habitat_subtype\tHabitat_type\n')
for files in alltraitsfile:
    filedir, filename = os.path.split(files)
    if '.fasta.16S.txt' not in filename:
        files16S = check_file(glob.glob(os.path.join(args.i16,filename.replace(args.f,args.f16))))
        MG = filename.split('.')[0]
        try:
            temp = MG+'\t'+'\t'.join(copycal(files, files16S,genenum))
            fout.write(temp+ '\t'+Meta[MG][0]+'\t'+Meta[MG][1]+'\n')
        except KeyError:
            fout.write(temp+'\n')
fout.close()

#for files in alltraitsfile:
    #filedir, filename = os.path.split(files)
    #traits_filter(files, files+'.aa')

#seqlist=dict()
#allseqlist=dict()
#for files in alltraitsfile:
    #traits_dedupli2(seqlist, allseqlist, files, files + '.aa.90.0.filter')
    #traits_dedupli(allseqlist,files)

#traits_habitat(args.b,allseqlist)



import argparse
import os
from Bio import SeqIO


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-db",
                    help="file name of your input database",
                     type=str, default='Butyrate.pro.aa',metavar='database.aa')
parser.add_argument("-dbf",
                    help="sequence format of your input database\
                    (1: nucleotide; 2: protein), \
                    (default \'1\' for nucleotide)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='.blast.txt',metavar='.blast.txt')
parser.add_argument('-s',
                    help="set the method to search the your database \
                    (1: blast; 2: hmm), \
                    (default \'1\' for blast search)",
                    metavar="1 or 2",
                    choices=[1, 2],
                    action='store', default=1, type=int)
# optional parameters
# optional search parameters
parser.add_argument('--id',
                    default=80.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based identity cutoff for blast (default is 60.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--ht',
                    default=80.0, action='store', type=float, metavar='60.0',
                    help='Optional: set the amno acid based hit-length cutoff for blast (default is 60.0)\n'
                         'Leave it alone if hmm is used')
parser.add_argument('--e',
                    default=1e-5, action='store', type=float, metavar='1e-5',
                    help='Optional: set the evalue cutoff for blast or hmm (default is 1e-5)')

################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def Calculate_length(file_name):
    DB_length=dict()
    try:
        for lines in open(file_name + '.length', 'r'):
            DB_length.setdefault(str(lines.split('\t')[0]), len(str(lines.split('\t')[-1]).replace('\n','')))
    except (IOError):
        Fasta_name = open(file_name, 'r')
        f = open(file_name + '.length', 'w')
        for record in SeqIO.parse(Fasta_name, 'fasta'):
            f.write(str(record.id) + '\t'  + str(
                len(record.seq)) + '\n')
            DB_length.setdefault(str(record.id), len(str(record.seq)))
        f.close()
    return DB_length

def compare_blast(line1,line2):
    ID1=float(str(line1).split('\t')[2])
    ID2=float(str(line2).split('\t')[2])
    Loci1=(float(str(line1).split('\t')[6])+float(str(line1).split('\t')[7]))/2.0
    Loci2=(float(str(line2).split('\t')[6])+float(str(line2).split('\t')[7]))/2.0
    if abs(Loci1 - Loci2) > 500 or ID1 == ID2:
        return [line1,line2]
    elif ID1 > ID2:
        return [line1]
    elif ID2 > ID1:
        return [line2]


def blast_list(file, Cutoff_identity,Cutoff_hitlength):
    f1=open(file+'.filter','w')
    All_hit = dict()
    if args.dbf == 1:
        ratio = 1.0
    else:
        ratio = 3.0
    for line in open(file, 'r'):
        if float(str(line).split('\t')[2]) >= Cutoff_identity:
            try:
                if float(str(line).split('\t')[3]) >= Cutoff_hitlength * 100.0/ratio/100.0:
                    if str(line).split('\t')[0] not in All_hit:
                        All_hit.setdefault(str(line).split('\t')[0],
                        [line])
                    else:
                        All_hit[str(line).split('\t')[0]] = compare_blast(All_hit[str(line).split('\t')[0]][0],
                        line)
            except TypeError:
                print (str(line).split('\t')[1])
    for genes in All_hit:
        for hits in All_hit[genes]:
            f1.write(hits)
    f1.close()


################################################### Programme #######################################################
if args.s == 1:
    DB_length = dict()
    if args.ht > 0.0:
        DB_length = Calculate_length(args.db)
    blast_list(os.path.join(args.i, args.f), float(args.id), float(args.ht))
else:
    pass



import os
from Bio import SeqIO
import argparse
import glob
import statistics

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-t",
                    help="trait name", type=str, default='ARG',metavar='trait name')
parser.add_argument("-i",
                    help="input dir of search results", type=str,
                    default='/scratch/users/anniz44/Metagenomes/ARG/search_output/0',metavar='.')
parser.add_argument("-f",
                    help="input format of searching file", type=str,
                    default='.blast.txt.filter',metavar='.blast.txt.filter')
parser.add_argument("-c",
                    help="input cell number summary", type=str,
                    default='/scratch/users/anniz44/Metagenomes/summary/cell.num.all.txt',metavar='cell.num.all.txt')
parser.add_argument("-16S",
                    help="input cell number summary", type=str,
                    default='/scratch/users/anniz44/Metagenomes/summary/16S.num.all.txt',metavar='16S.num.all.txt')
parser.add_argument("-m",
                    help="file name of trait length mapping file", type=str,
                    default='/scratch/users/anniz44/scripts/database/SARG.db.fasta.length',
                    metavar='db.fasta.length')

# optional  setup
parser.add_argument("--i16",
                    help="input dir of 16S search results", type=str,
                    default='/scratch/users/anniz44/Metagenomes/16S/0',metavar='.')
parser.add_argument("--f16",
                    help="input format of 16S searching file", type=str,
                    default='.16S.txt',metavar='.16S.txt')

parser.add_argument("--r",
                    help="output directory or folder of your results",
                    type=str, default='/scratch/users/anniz44/Metagenomes/summary',
                    metavar='Result')


################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass


################################################### Function ########################################################
def check_16S(inputfile):
    try:
        coverage=0
        for lines in open(os.path.join(args.i16,inputfile.replace(args.f,args.f16))):
            coverage += float(lines.split('\t')[3])/1500.0
        return coverage
    except IOError:
        return 0


def check_traits(inputfile):
    output = dict()
    for lines in open(os.path.join(args.i, inputfile)):
        gene = lines.split('\t')[1]
        maplength = float(lines.split('\t')[3])
        genelength = Mapping[gene]
        if gene not in output:
            output.setdefault(gene, maplength / genelength)
        else:
            output[gene] += maplength / genelength
    return output


################################################### Programme #######################################################
# load all files
searchfiles=glob.glob(os.path.join(args.i,'*'+args.f))

# input cell number
Cellnum=dict()
for lines in open(args.c,'r'):
    cellnum = []
    for numbers in lines.split('\t')[1:]:
        cellnum.append(float(numbers))
    averagecell = statistics.mean(cellnum)
    Cellnum.setdefault(lines.split('\t')[0].replace('.fasta','').replace('.blast.txt.filter',''),averagecell)

# input 16S number
Cell16S=dict()
for lines in open(args.c,'r'):
    cellnum = []
    averagecell = 0
    for numbers in lines.split('\t')[1:]:
        if float(numbers) > 0:
            cellnum.append(float(numbers))
    averagecell = statistics.mean(cellnum)
    if averagecell > 0:
        Cell16S.setdefault(lines.split('\t')[0].replace('.fasta','').replace('.blast.txt.filter',''),averagecell)

# gene ID: length
Mapping = dict()
for lines in open(args.m, 'r'):
    Mapping.setdefault(lines.split('\t')[0],
                       float(lines.split('\t')[1].split('\r')[0].split('\n')[0]))

# process all traits
#Cell16S=dict()
Trait=dict()
for searchfile in searchfiles:
    filedir, filename = os.path.split(searchfile.split('\r')[0].split('\n')[0])
    # check 16S
    #if filename.split('_1')[0].split('_2')[0] not in Cell16S:
    #    Cell16S.setdefault(filename.split('_1')[0].split('_2')[0],[])
    #Cell16S[filename.split('_1')[0].split('_2')[0]].append(check_16S(filename))
    # calculate trait
    filename_short = filename.split('_1')[0].split('_2')[0].replace('.fasta','').replace('.blast.txt.filter','')
    if filename_short not in Trait and '_1_2' not in filename:
        Trait.setdefault(filename_short,[])
    Tempset=[]
    Tempoutput = check_traits(filename)
    for traits in Mapping:
        try:
            Tempset.append(Tempoutput[traits])
        except KeyError:
            Tempset.append(0)
    Trait[filename_short].append(Tempset)


# output results
#f1=open(os.path.join(args.r,'16S.num.all.txt'),'w')
#for filename in Cell16S:
#    f1.write(filename)
#    for number in Cell16S[filename]:
#        f1.write('\t'+str(number))
#    f1.write('\n')
#f1.close()


# summarize result
fout1 = open(os.path.join(args.r,args.t+'.summary.16S.txt'),'w')
fout2 = open(os.path.join(args.r,args.t+'.summary.cell.txt'),'w')
fout3 = open(os.path.join(args.r,args.t+'.summary.copy.txt'),'w')
fout1.write('SampleID')
fout2.write('SampleID')
fout3.write('SampleID')
for traits in Mapping:
    fout1.write('\t'+traits)
    fout2.write('\t' + traits)
    fout3.write('\t' + traits)
fout1.write('\n')
fout2.write('\n')
fout3.write('\n')
for samples in Trait:
    if len(Trait[samples]) > 1:
        Tempset = []
        i = 0
        while i < len(Trait[samples][0]):
            Tempset.append((Trait[samples][0][i]+Trait[samples][1][i])/2.0)
            i+=1
        Trait[samples]=Tempset
    else:
        Trait[samples]=Trait[samples][0]
    if samples in Cell16S:
        fout1.write(samples)
        for numbers in Trait[samples]:
            fout1.write('\t'+str(numbers/Cell16S[samples]))
        fout1.write('\n')
    if samples in Cellnum:
        fout2.write(samples)
        for numbers in Trait[samples]:
            fout2.write('\t'+str(numbers/Cellnum[samples]))
        fout2.write('\n')
    fout3.write(samples)
    for numbers in Trait[samples]:
        fout3.write('\t' + str(numbers))
    fout3.write('\n')

fout1.close()
fout2.close()
fout3.close()



from setuptools import setup

setup(
    name="traits_finder",
    packages=['traits_finder'],
    version="1.2",
    description="search and summarize traits in genomes and metagenomes",
    author='Anni Zhang',
    author_email='anniz44@mit.edu',
    url='https://github.com/caozhichongchong/traits_finder',
    keywords=['metagenomes', 'genomes', 'function', 'traits'],
    license='MIT',
    install_requires=[
    'biopython',
    'argparse',
    'glob2',
    'statistics'
    ],
    include_package_data=True,
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    package_dir={'traits_finder': 'traits_finder'},
    package_data={'traits_finder': ['scripts/*','database/*','*.py']},
    entry_points={'console_scripts': ['traits_finder = traits_finder.__main__:main']},
    classifiers=[
        #'Development Status :: 1 - Alpha',
        #'Intended Audience :: Bioinformatics and Researchers',
        #'License :: MIT',
        #'Operating System :: MacOS',
        #'Operating System :: Microsoft :: Windows',
        #'Operating System :: LINUX',
        'Programming Language :: Python :: 3',
        #'Topic :: Antibiotic resistance :: risk ranking',
        #'Topic :: Metagenomes :: Antibiotic resistance',
    ]
)



import argparse
from Bio import SeqIO
import os

############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-i",
                    help="input dir", type=str, default='.',metavar='current dir (.)')
parser.add_argument("-f",
                    help="input filename", type=str, default='input.faa',metavar='input.faa')
parser.add_argument("-n",
                    help="prefix name for usearch result", type=str, default='.usearch.txt',metavar='.usearch.txt')
parser.add_argument("-r",
                    help="output dir", type=str, default='.',metavar='current dir (.)')


################################################## Definition ########################################################
args = parser.parse_args()


################################################### Function #######################################################
def Extractaa(root, searchfile, orffile, resultdir):
    # extract the query aa sequences according to a usearch or diamond alignment output
    # generate a smaller data of potential intI1 or sul1 for blastp search
    # input the query ORF sequences
    AA_seq = dict()
    try:
        for record in SeqIO.parse(open(os.path.join(root, orffile), 'r'), 'fasta'):
            AA_seq.setdefault(str(record.id), str(record.seq))
        f1 = open(os.path.join(resultdir, searchfile + '.aa'), 'w')
        try:
            for line in open(os.path.join(resultdir, searchfile), 'r'):
                try:
                    AA = str(line).split('\t')[0].split(' ')[0]
                    if AA_seq[AA] != '':
                        # avoid duplicate ORF
                        f1.write('>' + AA + '\n' +
                                 str(AA_seq[AA]) + '\n')
                        AA_seq[AA]=''
                except KeyError:
                    print ('AA not found for ' + AA)
                    flog.write('AA not found for ' + AA + '\n')
        except IOError:
            pass
        f1.close()
    except (IOError):
        print ('Files were missing: ' + orffile)


################################################### Programme #######################################################
Extractaa( args.i, args.f+args.n, args.f, args.r)


import os
from Bio import SeqIO
import argparse
import glob
import statistics
import traits_finder
import sys

################################################### Decalration #######################################################
print ("\
------------------------------------------------------------------------\n\
traits_finder searches and summarizes traits in genomes and metagenomes\n\
input: reference database and folder of genomes/metagenomes\n\
requirement: blast \n\n\
optional: diamond, bwa, hs-blastn, usearch \n\n\
Copyright:An Ni Zhang, Prof. Eric Alm, MIT\n\n\
Citation:\n\
Contact anniz44@mit.edu\n\
------------------------------------------------------------------------\n\
")

def main():
    usage = ("usage: traits_finder -t your.otu.table -s your.otu.seqs")
    version_string = 'traits_finder {v}, on Python {pyv[0]}.{pyv[1]}.{pyv[2]}'.format(
        v=traits_finder.__version__,
        pyv=sys.version_info,
    )
    ############################################ Arguments and declarations ##############################################
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-db",
                        help="file name of your input database",
                        type=str, default='Butyrate.pro.aa',metavar='database.aa')
    parser.add_argument("-dbf",
                        help="sequence format of your input database\
                        (1: nucleotide; 2: protein), \
                        (default \'1\' for nucleotide)",
                        metavar="1 or 2",
                        choices=[1, 2],
                        action='store', default=1, type=int)
    parser.add_argument("-m",
                        help="mapping file of traits to function", type=str,
                        default='Butyrate.pro.mapping.txt',
                        metavar='Butyrate.pro.mapping.txt')
    parser.add_argument("-i",
                        help="input folder of metagenomes or genomes", type=str,
                        default='.',metavar='current dir (.)')
    parser.add_argument("-inf",
                        help="input format\
                        (1: genomes; 2: metagenomes), \
                        (default \'1\' for genomes)",
                        metavar="1 or 2",
                        choices=[1, 2],
                        action='store', default=1, type=int)
    parser.add_argument("-fa",
                        help="input format of genome/metagenome sequence",
                        type=str, default='.fa', metavar='.fasta, .fna, .fastq or .fa')
    parser.add_argument('-s',
                        help="set the method to search the your database \
                        (1: blast; 2: hmm), \
                        (default \'1\' for blast search)",
                        metavar="1 or 2",
                        choices=[1, 2],
                        action='store', default=1, type=int)
    # optional parameters
    parser.add_argument("--l",
                        help="input list of metagenomes", type=str,
                        default='None',metavar='rep_metagenomes.txt')
    parser.add_argument("--meta",
                        help="metadata  of metagenomes", type=str,
                        default='None',
                        metavar='metadata.metagenomes.txt')
    parser.add_argument("--orf",
                        help="input format of genomes orfs", type=str, default='.genes.faa',metavar='.faa')
    # optional output setup
    parser.add_argument("--r",
                        help="output directory or folder of your results",
                        type=str, default='Result',metavar='Result')
    parser.add_argument("--r16",
                        help="output directory or folder of your 16S sequences",
                        type=str, default='Result',metavar='Result')
    # optional search parameters
    parser.add_argument('--t',
                        help="Optional: set the thread number assigned for running XXX (default 1)",
                        metavar="1 or more", action='store', default=1, type=int)
    parser.add_argument('--id',
                        default=75.0, action='store', type=float, metavar='60.0',
                        help='Optional: set the amno acid based identity cutoff for blast (default is 80.0)\n'
                             'Leave it alone if hmm is used')
    parser.add_argument('--ht',
                        default=75.0, action='store', type=float, metavar='60.0',
                        help='Optional: set the amno acid based hit-length cutoff for blast (default is 80.0)\n'
                             'Leave it alone if hmm is used')
    parser.add_argument('--e',
                        default=1e-2, action='store', type=float, metavar='1e-5',
                        help='Optional: set the evalue cutoff for blast or hmm (default is 1e-5)')
    # requirement for software calling
    parser.add_argument('--u',
                        help="Optional: use two-step method for blast search,"+
                             " \'None\' for using one step, \'usearch\' or \'diamond\' for using two-step \
                             (complete path to usearch or diamond if not in PATH, \
                             please make sure the search tools can be directly called), (default: \'None\')",
                        metavar="None or usearch",
                        action='store', default='None', type=str)
    parser.add_argument('--hmm',
                        help="Optional: complete path to hmmscan if not in PATH,",
                        metavar="/usr/local/bin/hmmscan",
                        action='store', default='hmmscan', type=str)
    parser.add_argument('--bp',
                        help="Optional: complete path to blastp or blastn if not in PATH, \'None\' for no blast search",
                        metavar="/usr/local/bin/blastp",
                        action='store', default='blastp', type=str)
    parser.add_argument('--bwa',
                        help="Optional: complete path to bwa if not in PATH,",
                        metavar="/usr/local/bin/bwa",
                        action='store', default='None', type=str)
    ################################################## Definition ########################################################
    args = parser.parse_args()
    workingdir=os.path.abspath(os.path.dirname(__file__))
    ################################################### Programme #######################################################
    f1 = open ('traits_finder.log','w')
    if args.inf == 1:
        cmd = ('python '+workingdir+'/Traits_WG.py -db %s -dbf %s -i %s -s %s --fa %s --orf %s --r %s --r16 %s --t %s --id %s --ht %s --e %s --u %s --hmm %s --bp %s --bwa %s\n'
        % (str(args.db),str(args.dbf),str(args.i),str(args.s),str(args.fa),str(args.orf),str(args.r),str(args.r16),str(args.t),str(args.id),str(args.ht),str(args.e),str(args.u),str(args.hmm),str(args.bp),str(args.bwa)))
        #cmd += ('python '+workingdir+'/scripts/Traits_summary_WG.py -t %s -db %s --fa %s --orf %s -i %s -m %s --r %s --r16 %s --s %s -c %s\n'
        #%(str(os.path.split(args.db)[1]),str(args.db),str(args.fa),str(args.orf),str(args.i),str(args.m),str(args.r),str(args.r16),str(os.path.join(args.r,'summary')),str(args.id)))
        f1.write(cmd)
        os.system(cmd)
    else:
        cmd = ('python '+workingdir+'/Traits_MG.py -db %s -dbf %s -i %s -s %s --fa %s -l %s --r %s --r16 %s --t %s --id %s --ht %s --e %s --u %s --hmm %s --bp %s --bwa %s\n'
        % (str(args.db),str(args.dbf),str(args.i),str(args.s),str(args.fa),str(args.l),str(args.r),str(args.r16),str(args.t),str(args.id),str(args.ht),str(args.e),str(args.u),str(args.hmm),str(args.bp),str(args.bwa)))
        #if args.s == 1:
        #   cmd += ('python '+workingdir+'/Copynumber_traits.py -i %s -i16 %s -o %s -f %s.blast.txt -f16 %s.16S.txt -l %s.length -tf %s -m %s -c %s'
        #    % (str(os.path.join(args.r,'search_output/0'))),str(args.r16),str(os.path.join(args.r,'summary'))),str(args.fa),str(args.fa),str(args.db),str(args.m),str(args.meta),str(args.id))
        #else:
            #cmd += ('python '+workingdir+'/Copynumber_traits_hmm.py -i %s -i16 %s -o %s -f %s.hmm2.txt -f16 %s.16S.txt -l %s.length -tf %s -m %s -c %s'
            #% (str(os.path.join(args.r,'search_output/0'))),str(args.r16),str(os.path.join(args.r,'summary'))),str(args.fa),str(args.fa),str(args.db),str(args.m),str(args.meta),str(args.e))
        #cmd += ('python '+workingdir+'/scripts/Traits_summary_MG.py -t %s -db %s --fa %s --orf %s -i %s -m %s --r %s --r16 %s --s %s -c %s\n'
        #%(str(os.path.split(args.db)[1]),str(args.db),str(args.fa),str(args.orf),str(args.i),str(args.m),str(args.r),str(args.r16),str(os.path.join(args.r,'summary')),str(args.id)))
        f1.write(cmd)
        os.system(cmd)

################################################## Function ########################################################

if __name__ == '__main__':
    main()
